import datetime
from typing import Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="GovernanceClassification")


@_attrs_define
class GovernanceClassification:
    """
    Attributes:
        id (str):
        name (str):
        description (str):
        requirement_ids (List[str]):
        created_by (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
    """

    id: str
    name: str
    description: str
    requirement_ids: List[str]
    created_by: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        description = self.description

        requirement_ids = self.requirement_ids

        created_by = self.created_by

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
                "description": description,
                "requirementIds": requirement_ids,
                "createdBy": created_by,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        description = d.pop("description")

        requirement_ids = cast(List[str], d.pop("requirementIds"))

        created_by = d.pop("createdBy")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        governance_classification = cls(
            id=id,
            name=name,
            description=description,
            requirement_ids=requirement_ids,
            created_by=created_by,
            created_at=created_at,
            updated_at=updated_at,
        )

        governance_classification.additional_properties = d
        return governance_classification

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())
